--
-- ConcreteSaw
--
-- @author  Stefan Geiger
-- @date  18/03/10
--
-- Copyright (C) GIANTS Software GmbH, Confidential, All Rights Reserved.

ConcreteSaw = {};

function onCreateTest(id)
    print("oCreate was called "..getName(id));
end;
g_onCreateUtil.addOnCreateFunction("onCreateTest", onCreateTest);


function ConcreteSaw.prerequisitesPresent(specializations)
    return true;
end;

function ConcreteSaw:load(xmlFile)
    self.raycastConcreteSawCallback = DeviceSpecializationUtil.callSpecializationsFunction("raycastConcreteSawCallback");

    self.concreteSawWheel = Utils.indexToObject(self.deviceNode, getXMLString(xmlFile, "device.concreteSaw#wheelIndex"));
    self.concreteSawWheelRotSpeed = Utils.getNoNil(getXMLFloat(xmlFile, "device.concreteSaw#wheelRotSpeed"), 1)*0.001;
    self.concreteSawWheelRotSpeedAcceleration = Utils.getNoNil(getXMLFloat(xmlFile, "device.concreteSaw#wheelRotSpeedAcceleration"), 0.01)*0.001*0.001;
    self.currentConcreteSawWheelSpeed = 0;
    self.concreteSawMaxDist = Utils.getNoNil(getXMLFloat(xmlFile, "device.concreteSaw#maxDist"), 1);
    self.concreteSawDamagePerMS = Utils.getNoNil(getXMLFloat(xmlFile, "device.concreteSaw#damagePerSecond"), 1)*0.001;
    self.concreteSawForce = Utils.getNoNil(getXMLFloat(xmlFile, "device.concreteSaw#force"), 0.5);

    self.concreteSawHitParticleSystem = {};
    Utils.loadParticleSystem(xmlFile, self.concreteSawHitParticleSystem, "device.concreteSawHitParticleSystem", self.deviceNode, false, nil, self.baseDirectory);

    -- this is a sample how two load an i3d file, with onCreate support for mods
    g_onCreateUtil.activateOnCreateFunctions();
    Utils.loadSharedI3DFile("test.i3d", self.baseDirectory, true);
    g_onCreateUtil.deactivateOnCreateFunctions();

end;

function ConcreteSaw:delete()
    Utils.deleteParticleSystem(self.concreteSawHitParticleSystem);
    ConcreteSaw.disableVibration(self);
end;

function ConcreteSaw:disableVibration()
    if (self.isVibrating) then
        self.isVibrating = false;
        local gamepadDeviceId = InputBinding.getGamepadDeviceIdOfDigitalAction(InputBinding.ACTIVATE_HANDTOOL);
        if (gamepadDeviceId) then
            setGamepadVibration(0.0, 0.0, -1, gamepadDeviceId);
        end;
    end;
end;

function ConcreteSaw:update(dt)
    if self:getIsActive() then
        local speedTarget = 0;
        if self:getIsActiveForInput() and InputBinding.isPressed(InputBinding.ACTIVATE_HANDTOOL) and (not self.takeInHandActive) then
            speedTarget = self.concreteSawWheelRotSpeed;

            if not self.idleSoundPlaying then
                playSample(self.idleSound, 0, 1, 0);
                self.idleSoundPlaying = true;
            end;

			local hasHitLastFrame = self.hasHit;

            -- Simulate shutter with random translation
            setTranslation(self.deviceNode, 0, 0, math.random(0,8)*0.0003);

            local x1, y1, z1 = localToWorld(Player.camera, 0, 0, -0.1);
            local dirX, dirY, dirZ = localDirectionToWorld(Player.camera, 0, 0, -1);
            self.concreteSawDirX = dirX;
            self.concreteSawDirY = dirY;
            self.concreteSawDirZ = dirZ;
            self.hasHit = false;
            raycastClosest(x1, y1, z1, dirX, dirY, dirZ, "raycastConcreteSawCallback", self.concreteSawMaxDist, self, 32);
            if self.hasHit then
                if not self.hitSoundPlaying then
                    playSample(self.hitSound, 0, 1, 0);
                    self.hitSoundPlaying = true;
                	setSamplePitch(self.idleSound, 1.75);
					setSampleVolume(self.idleSound, 1);
				end;

                Utils.setEmittingState(self.concreteSawHitParticleSystem, true);
            else
                stopSample(self.hitSound);
				setSamplePitch(self.idleSound, 1);
				setSampleVolume(self.idleSound, 0.5);
                self.hitSoundPlaying = false;
                Utils.setEmittingState(self.concreteSawHitParticleSystem, false);
            end;


			if ((not self.isVibrating) or (hasHitLastFrame ~= self.hasHit)) then
				ConcreteSaw.disableVibration(self);

				self.isVibrating = true;
				local gamepadDeviceId = InputBinding.getGamepadDeviceIdOfDigitalAction(InputBinding.ACTIVATE_HANDTOOL);
				local vibrationStrength = self.hasHit and 0.35 or 0.125;
				if (gamepadDeviceId) then
					setGamepadVibration(vibrationStrength, vibrationStrength, 24*60*60*1000, gamepadDeviceId);
				end;
			end;


            -- raise exhaustion level a bit, leads to higher bobbing amplitude
            Player.exhaustion = math.min(1, Player.exhaustion + 0.0001 * dt);
        else
			ConcreteSaw.disableVibration(self);

            stopSample(self.idleSound);
            self.idleSoundPlaying = false;
            stopSample(self.hitSound);
            self.hitSoundPlaying = false;
            Utils.setEmittingState(self.concreteSawHitParticleSystem, false);
        end;
        if self.concreteSawWheel ~= nil then
            if speedTarget < self.currentConcreteSawWheelSpeed then
                self.currentConcreteSawWheelSpeed = math.max(self.currentConcreteSawWheelSpeed - self.concreteSawWheelRotSpeedAcceleration*dt, speedTarget);
            else
                self.currentConcreteSawWheelSpeed = math.min(self.currentConcreteSawWheelSpeed + self.concreteSawWheelRotSpeedAcceleration*dt, speedTarget);
            end;
            rotate(self.concreteSawWheel, self.currentConcreteSawWheelSpeed*dt, 0, 0);
        end;
    end;
end;

function ConcreteSaw:onActivate()
end;

function ConcreteSaw:onDeactivate()
    stopSample(self.idleSound);
    self.idleSoundPlaying = false;
    stopSample(self.hitSound);
    self.hitSoundPlaying = false;
    Utils.setEmittingState(self.concreteSawHitParticleSystem, false);
end;

function ConcreteSaw:raycastConcreteSawCallback(transformId, x, y, z, distance, nx, ny, nz, subShapeIndex)
    if transformId ~= 0 then
        if subShapeIndex >= 0 then
            if getHasClassId(transformId, ClassIds.DESTRUCTION_SHAPE) then
                self.hasHit = true;
                addDestructionShapeDamage(transformId, subShapeIndex, self.concreteSawDamagePerMS*self.currentDt);
                local scale = self.concreteSawForce*getDestructionShapeMass(transformId, subShapeIndex);
                addDestructionShapeImpulse(transformId, subShapeIndex, self.concreteSawDirX*scale, self.concreteSawDirY*scale, self.concreteSawDirZ*scale, x, y, z, false);
            end;
        else
            local scale = 50;
            addForce(transformId, self.concreteSawDirX*scale, self.concreteSawDirY*scale, self.concreteSawDirZ*scale, x, y, z, false);
        end;
    end;
    return false;
end;